@@ -23,6 +23,8 @@
   @NOTATION@
 */
 
+#define _XOPEN_SOURCE 500
+
 #include <config.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -32,7 +34,7 @@
 #include <errno.h>
 
 /* Must be before all other gnome includes!! */
-#include "gnome-i18nP.h"
+#include <glib/gi18n-lib.h>
 
 #include "gnome-client.h"
 #include "gnome-uidefs.h"
@@ -852,6 +854,7 @@ client_interact_callback (SmcConn smc_conn, SmPointer client_data)
 /*****************************************************************************/
 /* Managing the master client */
 
+#if 0
 /* The following environment variables will be set on the master
    client, if they are defined the programs environment.  The array
    must end with a NULL entry.
@@ -863,6 +866,7 @@ static char* master_environment[]=
 {
   NULL
 };
+#endif
 
 /********* gnome_client module */
 
@@ -872,11 +876,6 @@ static void client_parse_func (poptContext ctx,
 			       const struct poptOption *opt,
 			       const char *arg, void *data);
 
-static gboolean gnome_client_goption_sm_client_id (const gchar *option_name,
-						   const gchar *value,
-						   gpointer data,
-						   GError **error);
-
 static gboolean gnome_client_goption_sm_config_prefix (const gchar *option_name,
 						       const gchar *value,
 						       gpointer data,
@@ -890,41 +889,7 @@ static gboolean gnome_client_goption_sm_disable (const gchar *option_name,
 static void gnome_client_pre_args_parse(GnomeProgram *app, GnomeModuleInfo *mod_info);
 static void gnome_client_post_args_parse(GnomeProgram *app, GnomeModuleInfo *mod_info);
 
-/* Command-line arguments understood by this module.  */
 enum { ARG_SM_CLIENT_ID=1, ARG_SM_CONFIG_PREFIX, ARG_SM_DISABLE };
-static const struct poptOption options[] = {
-  {NULL, '\0', POPT_ARG_INTL_DOMAIN, GETTEXT_PACKAGE, 0, NULL, NULL},
-
-  {NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_PRE | POPT_CBFLAG_POST,
-   client_parse_func, 0, NULL, NULL},
-
-  {"sm-client-id", '\0', POPT_ARG_STRING, NULL, ARG_SM_CLIENT_ID,
-   N_("Specify session management ID"), N_("ID")},
-
-  {"sm-config-prefix", '\0', POPT_ARG_STRING, NULL, ARG_SM_CONFIG_PREFIX,
-   N_("Specify prefix of saved configuration"), N_("PREFIX")},
-
-  {"sm-disable", '\0', POPT_ARG_NONE, NULL, ARG_SM_DISABLE,
-   N_("Disable connection to session manager"), NULL},
-
-  {NULL, '\0', 0, NULL, 0}
-};
-
-static const GOptionEntry session_goptions[] = {
-  { "sm-client-id", '\0', 0, G_OPTION_ARG_CALLBACK,
-    gnome_client_goption_sm_client_id, 
-    N_("Specify session management ID"), N_("ID")},
-
-  { "sm-config-prefix", '\0', 0, G_OPTION_ARG_CALLBACK,
-    gnome_client_goption_sm_config_prefix, 
-    N_("Specify prefix of saved configuration"), N_("PREFIX")},
-
-  { "sm-disable", '\0', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-    gnome_client_goption_sm_disable, 
-    N_("Disable connection to session manager"), NULL},
-
-  { NULL }
-};
 
 typedef struct {
 	guint sm_connect_id;
@@ -934,6 +899,8 @@ typedef struct {
         gboolean sm_connect;
 } GnomeProgramPrivate_gnome_client;
 
+static gchar *gnome_client_goption_sm_client_id = NULL;
+
 static GQuark quark_gnome_program_private_gnome_client = 0;
 static GQuark quark_gnome_program_class_gnome_client = 0;
 
@@ -1026,6 +993,18 @@ gnome_client_module_instance_init (GnomeProgram *program, GnomeModuleInfo *mod_i
 static GOptionGroup *
 gnome_client_module_get_goption_group (void)
 {
+	const GOptionEntry session_goptions[] = {
+		{ "sm-client-id", '\0', 0, G_OPTION_ARG_STRING,
+		  &gnome_client_goption_sm_client_id,
+		  N_("Specify session management ID"), N_("ID")},
+		{ "sm-config-prefix", '\0', 0, G_OPTION_ARG_CALLBACK,
+		  gnome_client_goption_sm_config_prefix,
+		  N_("Specify prefix of saved configuration"), N_("PREFIX")},
+		{ "sm-disable", '\0', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
+		  gnome_client_goption_sm_disable,
+		  N_("Disable connection to session manager"), NULL},
+		{ NULL }
+	};
 	GOptionGroup *option_group;
 
 	_gnome_ui_gettext_init (TRUE);
@@ -1043,6 +1022,25 @@ gnome_client_module_get_goption_group (void)
 const GnomeModuleInfo *
 gnome_client_module_info_get (void)
 {
+	/* Command-line arguments understood by this module.  */
+	static const struct poptOption options[] = {
+		{ NULL, '\0', POPT_ARG_INTL_DOMAIN, GETTEXT_PACKAGE, 0, NULL, NULL },
+
+		{ NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_PRE | POPT_CBFLAG_POST,
+		  client_parse_func, 0, NULL, NULL },
+
+		{ "sm-client-id", '\0', POPT_ARG_STRING, NULL, ARG_SM_CLIENT_ID,
+		  N_("Specify session management ID"), N_("ID") },
+
+		{ "sm-config-prefix", '\0', POPT_ARG_STRING, NULL, ARG_SM_CONFIG_PREFIX,
+		  N_("Specify prefix of saved configuration"), N_("PREFIX") },
+
+		{ "sm-disable", '\0', POPT_ARG_NONE, NULL, ARG_SM_DISABLE,
+		  N_("Disable connection to session manager"), NULL },
+
+		{ NULL, '\0', 0, NULL, 0 }
+	};
+
 	static GnomeModuleInfo module_info = {
 		"gnome-client", VERSION, N_("Session management"),
 		NULL, gnome_client_module_instance_init,
@@ -1103,17 +1101,6 @@ client_parse_func (poptContext ctx,
   }
 }
 
-static gboolean
-gnome_client_goption_sm_client_id (const gchar *option_name,
-				   const gchar *value,
-				   gpointer data,
-				   GError **error)
-{
-	gnome_client_set_id (master_client, value);
-
-	return TRUE;
-}
-
 static gboolean
 gnome_client_goption_sm_disable (const gchar *option_name,
 				 const gchar *value,
@@ -1142,7 +1129,9 @@ gnome_client_goption_sm_config_prefix (const gchar *option_name,
 static void
 gnome_client_pre_args_parse(GnomeProgram *app, GnomeModuleInfo *mod_info)
 {
+#if 0
   int i;
+#endif
   char *cwd;
 
   /* Make sure the Gtk+ type system is initialized.  */
@@ -1160,6 +1149,7 @@ gnome_client_pre_args_parse(GnomeProgram *app, GnomeModuleInfo *mod_info)
   /* Initialise ICE */
   gnome_ice_init ();
 
+#if 0
   /* Set the master client's environment.  */
   for (i= 0; master_environment[i]; i++)
     {
@@ -1170,6 +1160,7 @@ gnome_client_pre_args_parse(GnomeProgram *app, GnomeModuleInfo *mod_info)
 				      master_environment[i],
 				      value);
     }
+#endif
 
   cwd = g_get_current_dir();
   if (cwd != NULL)
@@ -1191,12 +1182,28 @@ static void
 gnome_client_post_args_parse(GnomeProgram *app, GnomeModuleInfo *mod_info)
 {
   gboolean do_connect = TRUE;
-  GValue value = { 0, };
 
-  g_value_init (&value, G_TYPE_BOOLEAN);
-  g_object_get_property (G_OBJECT (app), GNOME_CLIENT_PARAM_SM_CONNECT, &value);
-  do_connect = g_value_get_boolean (&value);
-  g_value_unset (&value);
+  g_object_get (G_OBJECT (app), GNOME_CLIENT_PARAM_SM_CONNECT, &do_connect, NULL);
+
+  if (gnome_client_goption_sm_client_id != NULL)
+    {
+      gnome_client_set_id (master_client, gnome_client_goption_sm_client_id);
+      g_free (gnome_client_goption_sm_client_id);
+      gnome_client_goption_sm_client_id = NULL;
+    }
+  else
+    {
+      const gchar *desktop_autostart_id;
+
+      desktop_autostart_id = g_getenv ("DESKTOP_AUTOSTART_ID");
+
+      if (desktop_autostart_id != NULL)
+        gnome_client_set_id (master_client, desktop_autostart_id);
+    }
+
+  /* Unset DESKTOP_AUTOSTART_ID in order to avoid child processes to
+   * use the same client id. */
+  g_unsetenv ("DESKTOP_AUTOSTART_ID");
 
   /* We're done, so we can connect to the session manager now.  */
   if (do_connect)
