@@ -23,7 +23,7 @@
 #include <libgnome/gnome-macros.h>
 
 /* Must be before all other gnome includes!! */
-#include "gnome-i18nP.h"
+#include <glib/gi18n-lib.h>
 
 #include <gtk/gtksignal.h>
 #include <gtk/gtkmain.h>
@@ -51,11 +51,11 @@ struct _GnomeFontPickerPrivate {
 
         int           use_font_in_label_size;
 
-        GnomeFontPickerMode mode : 2;
+        guint         mode : 2; /* GnomeFontPickerMode */
 
         /* Only for GNOME_FONT_PICKER_MODE_FONT_INFO */
-        gboolean      use_font_in_label : 1;
-        gboolean      show_size : 1;
+        guint      use_font_in_label : 1;
+        guint      show_size : 1;
 
 	GtkWidget     *font_dialog;
         GtkWidget     *inside;
